--[[
    Copyright (C) 2011 Omer Bahri Gordebak
    Using some of the code provided by:
    Copyright (c) 2010 Enrique García Cota

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Omer Bahri Gordebak <gordebak@gmail.com>
]]

local gr = love.graphics

function love.load()
	love.window.setMode( 608, 400, {fullscreen=false, fullscreentype="exclusive"} )
	love.window.setTitle( "Semicontrol" )
	player1		= gr.newImage( 'images/player1.png' )
	player2		= gr.newImage( 'images/player2.png' )
	player2r	= gr.newImage( 'images/playerR.png' )
	wall		= gr.newImage( 'images/wall.png' )
	goal		= gr.newImage( 'images/goal.png' )
	reverse		= gr.newImage( 'images/reverse.png' )
	splash		= gr.newImage( 'images/splash.png' )
	finished	= gr.newImage( 'images/finish.png' )
	deSellect	= gr.newImage( 'images/off.png' )
	nxLevel		= love.audio.newSource( 'sounds/levelup.wav' )
	music		= love.audio.newSource( 'sounds/sq-never-satisfied-music.wav' )
	music:setLooping( true )
	levels		= love.filesystem.getDirectoryItems( 'maps' )
	maps		= {}
	for i		= 1, #levels do maps[i] = 'maps/'..levels[i] end
	level		= nextLevel()
	maxLevels	= 10
	levelsPlayed= 0
	showSplash	= true
	end

function love.draw()
	if showSplash then
		gr.draw( splash, 0, 0 )
		if not playMusic then gr.draw( deSellect, 190, 280 ) end
		if not playSound then gr.draw( deSellect, 190, 344 ) end
		else
		if not isFinished then drawField() end
		end
	if isFinished then gr.draw( finished, 0, 0 ) end
	end

function love.update()
	if newLevel then newMap( level ) end
	end

function love.keypressed(key)
	if key == 'escape' and showSplash then
		love.event.push( 'quit' )
		end
	if key == 'escape' and not showSplash then
		if not isFinished then
			showSplash = true
			else
			love.event.push( 'quit' )
			end
		end
	if key == 'space' and showSplash and not isFinished then
		showSplash = false
		end
	if key == 'm' then
		if playMusic then
			music:stop()
			playMusic = false
			else
			love.audio.play( music )
			playMusic = true
			end
		end
	if key == 'f' then playSound = not playSound end
	if key == 'f11' then
		if not isfullscreen then
			love.window.setMode(640, 480, {fullscreen=true, fullscreentype="exclusive"})
			isfullscreen = true
			else
			love.window.setMode(640, 480, {fullscreen=false, fullscreentype="exclusive"})
			isfullscreen = false
			end
		end
	if key == 'left'	then move(-32,  0) end
	if key == 'right'	then move( 32,  0) end
	if key == 'up'		then move(  0,-32) end
	if key == 'down'	then move(  0, 32) end
	end

function love.mousepressed( x, y )
	if showSplash then
		if overButton( x, y, 65, 160, 225, 100 ) then
			showSplash = false
			end
		if overButton( x, y, 95, 270, 170,  55 ) then
			if playMusic then
				music:stop()
				playMusic = false
				else
				love.audio.play( music )
				playMusic = true
				end
			end
		if overButton( x, y, 95, 335, 170,  50 ) then
			playSound = not playSound
			end
		end
	end

function newMap( level )
	name		= maps[ level ]
	walls 		= {}
	rv			= { 0, 0 }
	isReversed 	= false
	local row	= 0
	for line in love.filesystem.lines( name ) do
		row = row + 1
		for col = 1, #line do
			local char = string.sub( line, col, col )
			if char == '#' then walls[#walls+1] = { 32*col, 32*row } end
			if char == 'X' then go = { 32 * col, 32 * row} end
			if char == '1' then p1 = { 32 * col, 32 * row} end
			if char == '2' then p2 = { 32 * col, 32 * row} end
			if char == 'R' then rv = { 32 * col, 32 * row} end
			end
		end
	if #levels > 1 then table.remove( levels, level ) end
	newLevel = false
	end

function drawWalls()
	for i = 1, #walls do gr.draw( wall, walls[i][1], walls[i][2] ) end
	end

function move( cx, cy )
	if isReversed then cx = -cx cy = -cy end
	local moveLeft, moveRight = true, true
	for i = 1, #walls do
		local x, y = walls[i][1], walls[i][2]
		if p1[2] + cy == y and p1[1] + cx == x then
			moveLeft = false
			else
			if p2[2] + cy == y and p2[1] + cx == x then
				moveRight = false
				end
			end
		end
	if p2[1] + cx == go[1] and p2[2] + cy == go[2] then
		levelsPlayed	= levelsPlayed+1
		if playSound then love.audio.play( nxLevel ) end
		if levelsPlayed == maxLevels then
			isFinished	= true
			else
			level		= nextLevel( )
			end
		end
	if moveLeft then p1[1]  = p1[1] + cx p1[2] = p1[2] + cy end
	if moveRight and moveLeft then
		p2[1] = p2[1] + cx
		p2[2] = p2[2] + cy
		if p2[1] == rv[1] and p2[2] == rv[2] then isReversed = true end
		end
	end

function overButton( mx, my, bx, by, bw, bh )
	return mx > bx and mx < bx+bw and my > by and my < by+bh
	end

function nextLevel( ) newLevel = true return love.math.random( #levels ) end

function drawField()
	drawWalls()
	gr.draw( player1, p1[1], p1[2] )
	if rv[1] > 0 then gr.draw( reverse, rv[1], rv[2] ) end
	if isReversed then
		gr.draw( player2r, p2[1], p2[2] )
		else
		gr.draw( player2, p2[1], p2[2] )
		end
	gr.draw( goal, go[1], go[2] )
	gr.print( string.sub( name, 6, #name-4 ), 32, 364 )
	end
